#include <iostream>
#include <netinet/tcp.h>

#include "ace/OS_NS_time.h"
#include "ace/Log_Msg.h"
#include "ace/SSL/SSL_Context.h"
#include "ace/SSL/SSL_SOCK_Stream.h"
#include "ace/SSL/SSL_SOCK_Connector.h"

static u_short SERVER_PORT = ACE_DEFAULT_SERVER_PORT;
static const char *const SERVER_HOST = ACE_DEFAULT_SERVER_HOST;
static const uint16_t BUF_SIZE = 1024;

int main (int argc, char *argv[])
{
    const char *server_host = argc > 1 ? argv[1] : SERVER_HOST;
    u_short server_port = argc > 2 ? ACE_OS::atoi (argv[2]) : SERVER_PORT;
    const char* certFile = argc > 3 ? argv[3] : "./server.pem";

    ACE_SSL_Context *context = ACE_SSL_Context::instance ();

    context->certificate(certFile, SSL_FILETYPE_PEM);
    //context->private_key ("./key.pem", SSL_FILETYPE_PEM);
    std::cout << "Certfile: " << certFile << std::endl;

    ACE_SSL_SOCK_Stream server;

    ACE_SSL_SOCK_Connector connector;
    ACE_INET_Addr addr(server_port, server_host);

    if (connector.connect(server, addr) == -1)
        ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "Connect failed"), -1);

    ACE_DEBUG((LM_INFO, "%D [%P] Connected to %C:%d\n", server_host, server_port));    

    uint32_t cnt = 0;
    while(1)    
    {
        char buf[BUFSIZ];

        /* Create our message with the message number */
        ACE_OS::sprintf (buf,
                "This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. This is a test message. End of test...");

        if (server.send_n (buf,
                    ACE_OS::strlen (buf)) == -1)
            ACE_ERROR_RETURN ((LM_ERROR,
                        "%p\n",
                        "send"),
                    -1);
        else {
            ACE_DEBUG((LM_DEBUG,"%D [%P] Send successful, iteration %d\n", ++cnt));
            uint8_t buf[BUF_SIZE];
            int sz = server.recv(buf, BUF_SIZE);
            ACE_DEBUG((LM_DEBUG,"%D [%P] Received %d bytes\n", sz));

            struct timespec requested;
            requested.tv_sec = 0;
            requested.tv_nsec = 1000000;
            ACE_OS::nanosleep(&requested);
        }
    }
    ACE_OS::sleep(3);

    if (server.close () == -1)
        ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "close"), -1);
    return 0;
}
