#if !defined __SERVER_H__
#define      __SERVER_H__

#include "ace/OS_main.h"
#include "ace/Log_Msg.h"
#include "ace/INET_Addr.h"
#include "ace/SOCK_Acceptor.h"
#include "ace/Reactor.h"
#include "ace/Acceptor.h"
#include "ace/Message_Block.h"
#include "ace/SOCK_Stream.h"
#include "ace/Svc_Handler.h"
#include "ace/SSL/SSL_Context.h"
#include "ace/SSL/SSL_SOCK_Acceptor.h"
#include "ace/SSL/SSL_SOCK_Stream.h"
#include "boost/shared_ptr.hpp"


#ifndef ACE_SSL_SOCK_STREAM
#if defined (ACE_HAS_TEMPLATE_TYPEDEFS)
#define ACE_SSL_SOCK_STREAM ACE_SSL_SOCK_Stream
#define ACE_SSL_SOCK_ACCEPTOR ACE_SSL_SOCK_Acceptor
#else
#define ACE_SSL_SOCK_STREAM ACE_SSL_SOCK_Stream, ACE_INET_Addr
#define ACE_SSL_SOCK_ACCEPTOR ACE_SSL_SOCK_Acceptor, ACE_INET_Addr
#endif /* ACE_HAS_TEMPLATE_TYPEDEFS */
#endif /* ACE_SSL_SOCK_STREAM */

class TLSServer;

struct Message {
    enum TransportType{
         NONE,
         TLS
    };

    Message();
    TransportType transportType_;
    TLSServer* tlsServer_;
    unsigned char  *data_;
    unsigned       len_;
    ACE_INET_Addr  peer_;

    int sendMessage(unsigned char* buf, unsigned len);

    void reset();
    Message(const Message& md);

 private:

};

class TLSServer : public ACE_Svc_Handler<ACE_SSL_SOCK_STREAM, ACE_NULL_SYNCH> {
typedef ACE_Svc_Handler<ACE_SSL_SOCK_STREAM, ACE_NULL_SYNCH>super;

public:
    class Sink {
     public:
        virtual ~Sink(){}
        virtual int appMessageSink(boost::shared_ptr<Message>msg) = 0;
    };

    TLSServer();
    virtual ~TLSServer();

    static int initTLS(const char* certificate, const char* privateKey);

    int open(void *ptr=0);

    virtual int handle_input(ACE_HANDLE fd = ACE_INVALID_HANDLE);

    virtual int handle_output(ACE_HANDLE fd = ACE_INVALID_HANDLE);

    virtual int handle_close(ACE_HANDLE fd, ACE_Reactor_Mask mask );

    virtual int handle_timeout (const ACE_Time_Value & tv, const void *arg = 0);

    int send(const unsigned char *buf, unsigned len);

    int recvMsg();

    int closeConnection() { return peer().close(); }

    int invokeCallback(boost::shared_ptr<Message>&m) {
        return sink_->appMessageSink(m);
    }
  
    static void setSink(Sink* sink);

    unsigned short getRemotePort() { 
        return client_.get_port_number();
    }

    std::string getRemoteIP(){
        return client_.get_host_addr();
    }

private:
    static TLSServer::Sink* sink_;
    uint8_t* buff_;
    Message  *mdata_;
    int*     incarnation_;
    long      thdl_;
    bool     inactivityFlg_;
    boost::shared_ptr<uint8_t> localBuffer_;
    unsigned      localBufferSize_;
    ACE_INET_Addr client_;
    struct sockaddr_in stclient_;
    bool connectionOpened_;
    Message::TransportType transportType_;
    //FIXME: To be removed
    uint32_t count_;
    uint32_t maxRdSize_;
};

typedef ACE_Acceptor<TLSServer, ACE_SSL_SOCK_ACCEPTOR> TLSService;

#endif
