#include "tls_server.h"

using namespace std;

// Constructor
TlsServerApp::TlsServerApp (int argc, char *argv[]) {
    if(argc > 1) {
        ipPort_ = argv[1];
    } else {
        ipPort_ = "0.0.0.0:4443";
    }

    ACE_LOG_MSG->priority_mask (LM_INFO | LM_ERROR | LM_WARNING,
                              ACE_Log_Msg::PROCESS);
    if((argc > 2) && (strcmp(argv[2], "debug") == 0)) {
        ACE_LOG_MSG->priority_mask (LM_DEBUG | LM_INFO | LM_ERROR | LM_WARNING,
                                    ACE_Log_Msg::PROCESS);
    }   

    std::string keyFile, certFile;
    if (argc > 3) {
        keyFile = argv[3];
    }
    if (argc > 4) {
        certFile = argv[4];
    }

    clientMsgSize_ = 0;
    if (argc > 5) {
        clientMsgSize_ = atoi(argv[5]);
    }
    initialize(keyFile, certFile);
}

// Destructor
TlsServerApp::~TlsServerApp() {
}

void
TlsServerApp::initialize (const std::string& keyFile, 
                          const std::string& certFile) {
    TLSServer::initTLS(certFile.c_str(), keyFile.c_str());

    ACE_INET_Addr addr(ipPort_.c_str());
    tlsserver_.reset(new TLSService());
    if (tlsserver_->open(addr) != 0) {
        ACE_DEBUG((LM_ERROR, "%D [%P] Unable to create TLS Listener"));
        exit(1);
    }

    ACE_DEBUG((LM_INFO, "%D [%P] TLS server is listening on [%C]\n", ipPort_.c_str()));

    TLSServer::setSink(this);
}

static const uint16_t EXPECTED_MSG_SIZE = 500;
int
TlsServerApp::appMessageSink(boost::shared_ptr<Message> md) {

    bool isTLS = (md->tlsServer_ != NULL);

    ACE_DEBUG((LM_DEBUG, 
        "%D [%P] Incoming message from %C:%d, Message size %d, Transport type %C\n", 
        md->peer_.get_host_addr(), md->peer_.get_port_number(), md->len_, "TLS"));
    ACE_HEX_DUMP((LM_DEBUG, (const char*)md->data_, (size_t)md->len_, " message "));

    if ((clientMsgSize_ && md->len_ == clientMsgSize_) ||
        (!clientMsgSize_ && md->len_ == EXPECTED_MSG_SIZE)) {
        unsigned char respMsg[] = "OK. Received it.";
        md->sendMessage(respMsg, (unsigned int)(sizeof(respMsg) + 1) );
        ACE_DEBUG((LM_DEBUG, "%D [%P] Response sent, returning %d bytes\n", md->len_));
        return md->len_; 
    }
    else {
        ACE_DEBUG((LM_INFO, "%D [%P] Partial message received, returning 0\n"));
        return 0;
    }
}

void
TlsServerApp::service () {
    ACE_Reactor::instance()->owner (ACE_Thread::self ());

    while (true) {
        ACE_Reactor::instance()->run_reactor_event_loop();
    } // While !Shuttingdown
}

int main (int argc, char *argv[]) {

    TlsServerApp tlsServerApp(argc, argv);

    tlsServerApp.service();

    ACE_Reactor::instance()->close_singleton();

    return 1;
}
